@@ -19,6 +19,7 @@ class Module:
 
     def __call__(self, x: Variable, *args, **kwargs):
         out = self.forward(x)
+        out.retain_grad()
         self.__add_trainable_variables(x)
         return out
 
@@ -176,7 +177,7 @@ class Module:
         raise NotImplemented
 
     def backward(self, loss: Variable):
-        loss.backward()
+        loss.grad_fn(loss)
         for layer in reversed(self.graph):
             layer.backward()
 
@@ -202,6 +203,8 @@ class Module:
         print('Layer(type)'.ljust(25), 'Output Shape'.ljust(20), 'Param'.ljust(10), 'Connected to'.ljust(15))
         print('#' * bar_nums)
         total_params = 0
+        if self.graph is None:
+            raise ValueError('Please compile Model!')
         for layer in self.graph:
             layer_name = '%s (%s)' % (layer.name, layer.__class__.__name__)
             params = layer.params_count()
